require 'rubygems'
require 'base64'
require 'openssl'
require 'sinatra'
require 'shopify_api'
require 'active_support/security_utils'
require 'rest-client'
require 'json'


# The Shopify app's shared secret, viewable from the Partner dashboard
SHARED_SECRET = 'SHARED_SECRET'

## REMOVE THESE BEFORE PUBLISHING BLOG/TUTORIAL
API_KEY = 'API_KEY'
PASSWORD = 'PASSWORD'
SHOP_NAME = 'SHOP_NAME'

shop_url = "https://#{API_KEY}:#{PASSWORD}@#{SHOP_NAME}.myshopify.com"
ShopifyAPI::Base.site = shop_url
ShopifyAPI::Base.api_version = '2019-04' 


helpers do
  # Compare the computed HMAC digest based on the shared secret and the request contents
  # to the reported HMAC in the headers
  def verify_webhook(data, hmac_header)
    calculated_hmac = Base64.strict_encode64(OpenSSL::HMAC.digest('sha256', SHARED_SECRET, data))
    unless ActiveSupport::SecurityUtils.secure_compare(calculated_hmac, hmac_header)
      return [403, 'Authorization failed.  Provided hmac was #{hmac_header}']
    end
  end
end


# Respond to HTTP POST requests sent to this web service
post '/webhook/validate_email' do
  request.body.rewind
  data = request.body.read
  verified = verify_webhook(data, env["HTTP_X_SHOPIFY_HMAC_SHA256"])
  json_data = JSON.parse data

  #Get email address from payload
  emailIn = json_data['email'];

  #Set license key here
  #Note: If using a production key update main endpoint to ws.serviceobjects.com and backup endpoint to wsbackup.serviceobjects.com
  licenseKey = "YOUR_LICENSEKEY";

  #Make call to Service Objects Web Service
  begin
    response = RestClient::Request.new(
      :method => :get,
      :url => 'https://trial.serviceobjects.com/ev3/web.svc/json/ValidateEmailAddress?EmailAddress=' + emailIn + '&AllowCorrections=false&Timeout=30000&LicenseKey=' + licenseKey,
      :verify_ssl => false
    ).execute
    
    puts response.to_str
    results = JSON.parse(response.to_str)

    #Check for null result object or fatal error from Service Objects Endpoint, raise exception and call backup endpoint if found
    if (results.nil? or (!results['Error'].nil? and results['Error']['TypeCode'] == '3')).
      raise exception.new 'Calling Failover Endpoint'
    end
  rescue => exception
    #If a production key is used, call wsbackup.serviceobjects.com... with a production license key
    #Here we will just recall the trial endpoint.
    response = RestClient::Request.new(
      :method => :get,
      :url => 'https://trial.serviceobjects.com/ev3/web.svc/json/ValidateEmailAddress?EmailAddress=' + emailIn + '&AllowCorrections=false&Timeout=30000&LicenseKey=' + licenseKey,
      :verify_ssl => false
    ).execute
    results = JSON.parse(response.to_str)
  end
  
  

  #Retrieve the customer id so the customer contact can be updated. 
  customer = ShopifyAPI::Customer.find(json_data['id'].to_i)

  #If there is no error object present, we can update the note field of the customer object with the API response
  if (results['Error'].nil?)
  
    score = results['ValidateEmailInfo']['Score']

    #If custom fields are being used, then this would be the place to assign and access them. The Fields in the email validation service can be accessed as follows
    # IsDeliverable = results['ValidateEmailInfo']['IsDeliverable']
    # EmailAddressIn = results['ValidateEmailInfo']['EmailAddressIn']
    # EmailAddressOut = results['ValidateEmailInfo']['EmailAddressOut']
    # EmailCorrected = results['ValidateEmailInfo']['EmailCorrected']
    # Box = results['ValidateEmailInfo']['Box']
    # Domain = results['ValidateEmailInfo']['Domain']
    # TopLevelDomain = results['ValidateEmailInfo']['TopLevelDomain']
    # TopLevelDomainDescription = results['ValidateEmailInfo']['TopLevelDomainDescription']
    # IsSMTPServerGood = results['ValidateEmailInfo']['IsSMTPServerGood']
    # IsCatchAllDomain  = results['ValidateEmailInfo']['IsCatchAllDomain ']
    # IsSMTPMailBoxGood = results['ValidateEmailInfo']['IsSMTPMailBoxGood']
    # WarningCodes  = results['ValidateEmailInfo']['WarningCodes ']
    # WarningDescriptions  = results['ValidateEmailInfo']['WarningDescriptions ']
    # NotesCodes  = results['ValidateEmailInfo']['NotesCodes ']
    # NotesDescription  = results['ValidateEmailInfo']['NotesDescription ']


    #If the score is less than 1, we'll consider this a good email. Otherwise it will be flagged as bad. 
    #This can be adjusted for specific business logic or additional fields listed above can also be levaraged from the service.
    if score.to_i <= 1
      customer.note += '[Good Email, Score: ' + score.to_s + ']'
    else
      customer.note += '[Bad Email, Score: ' + score.to_s + ']'
    end

  #If there is an error object, we'll update the notes section of the customer object. 
  else
    customer.note += '[DOTS Email Validation 3 Error: ' + results['Error']['Desc'] + ']'
  end

  #Save the customer object so the notes are  updated
  customer.save

  #Return a 200 HTTP code to let Shopify know the webhook processed successfully.
  return [200, 'Webhook successfully received']

  
end